@@ -5,24 +5,24 @@ module Content.ItemKindEmbed
 
 import Prelude ()
 
-import Game.LambdaHack.Common.Prelude
+import Game.LambdaHack.Core.Prelude
 
-import Game.LambdaHack.Common.Color
-import Game.LambdaHack.Common.Dice
-import Game.LambdaHack.Common.Flavour
-import Game.LambdaHack.Common.ItemAspect (Aspect (..))
-import Game.LambdaHack.Common.Misc
 import Game.LambdaHack.Content.ItemKind
+import Game.LambdaHack.Core.Dice
+import Game.LambdaHack.Definition.Ability
+import Game.LambdaHack.Definition.Color
+import Game.LambdaHack.Definition.Defs
+import Game.LambdaHack.Definition.Flavour
 
 embeds :: [ItemKind]
 embeds =
-  [scratchOnWall, obscenePictogram, subtleFresco, treasureCache, treasureCacheTrap, signboardExit, signboardEmbed, fireSmall, fireBig, frost, rubble, doorwayTrapTemplate, doorwayTrap1, doorwayTrap2, doorwayTrap3, stairsUp, stairsDown, escape, staircaseTrapUp, staircaseTrapDown, pulpit]
+  [scratchOnWall, obscenePictogram, subtleFresco, treasureCache, treasureCacheTrap, signboardExit, signboardEmbed, signboardMerchandise, fireSmall, fireBig, frost, rubble, doorwayTrapTemplate, doorwayTrap1, doorwayTrap2, doorwayTrap3, stairsUp, stairsDown, escape, staircaseTrapUp, staircaseTrapDown, pulpit, shallowWater, straightPath, frozenGround]
 
-scratchOnWall,    obscenePictogram, subtleFresco, treasureCache, treasureCacheTrap, signboardExit, signboardEmbed, fireSmall, fireBig, frost, rubble, doorwayTrapTemplate, doorwayTrap1, doorwayTrap2, doorwayTrap3, stairsUp, stairsDown, escape, staircaseTrapUp, staircaseTrapDown, pulpit :: ItemKind
+scratchOnWall,    obscenePictogram, subtleFresco, treasureCache, treasureCacheTrap, signboardExit, signboardEmbed, signboardMerchandise, fireSmall, fireBig, frost, rubble, doorwayTrapTemplate, doorwayTrap1, doorwayTrap2, doorwayTrap3, stairsUp, stairsDown, escape, staircaseTrapUp, staircaseTrapDown, pulpit, shallowWater, straightPath, frozenGround :: ItemKind
 
 -- Make sure very few walls are substantially useful, e.g., caches,
 -- and none that are secret. Otherwise the player will spend a lot of time
--- bumping walls, which is boring compare to fights or dialogues
+-- bumping walls, which is boring compared to fights or dialogues
 -- and ever worse, the player will bump all secret walls, wasting time
 -- and foregoing the fun of guessing how to find entrance to a disjoint part
 -- of the level by bumping the least number of secret walls.
@@ -36,10 +36,9 @@ scratchOnWall = ItemKind
   , iverbHit = "scratch"
   , iweight  = 1000
   , idamage  = 0
-  , iaspects = []
-  , ieffects = [ Temporary "start making sense of the scratches"
+  , iaspects = [SetFlag Durable]
+  , ieffects = [ VerbMsg "start making sense of the scratches"
                , Detect DetectHidden 3 ]
-  , ifeature = [Durable]
   , idesc    = "A seemingly random series of scratches, carved deep into the wall."
   , ikit     = []
   }
@@ -53,13 +52,11 @@ obscenePictogram = ItemKind
   , iverbHit = "infuriate"
   , iweight  = 1000
   , idamage  = 0
-  , iaspects = [Timeout 7]
-  , ieffects = [ Recharging $ Temporary "enter destructive rage at the sight of an obscene pictogram"
-               , Recharging $ RefillCalm (-20)
-               , Recharging $ OneOf
-                   [ toOrganGood "strengthened" (3 + 1 `d` 2)
-                   , CreateItem CInv "sandstone rock" timerNone ] ]
-  , ifeature = [Durable]
+  , iaspects = [Timeout 7, SetFlag Durable]
+  , ieffects = [ VerbMsg "enter destructive rage at the sight of an obscene pictogram"
+               , RefillCalm (-20)
+               , OneOf [ toOrganGood "strengthened" (3 + 1 `d` 2)
+                       , CreateItem CInv "sandstone rock" timerNone ] ]
   , idesc    = "It's not even anatomically possible."
   , ikit     = []
   }
@@ -73,22 +70,28 @@ subtleFresco = ItemKind
   , iverbHit = "sooth"
   , iweight  = 1000
   , idamage  = 0
-  , iaspects = [Timeout 7]
-  , ieffects = [ Temporary "feel refreshed by the subtle fresco"
-               , RefillCalm 2
-               , Recharging $ toOrganGood "far-sighted" (3 + 1 `d` 2)
-               , Recharging $ toOrganGood "keen-smelling" (3 + 1 `d` 2) ]
-  , ifeature = [Durable]
+  , iaspects = [Timeout 7, SetFlag Durable]
+  , ieffects = [ VerbMsg "feel refreshed by the subtle fresco"
+               , toOrganGood "far-sighted" (3 + 1 `d` 2)
+               , toOrganGood "keen-smelling" (3 + 1 `d` 2) ]
+                 -- hearing gets a boost through bracing, so no need here
   , idesc    = "Expensive yet tasteful."
   , ikit     = []
   }
-treasureCache = stairsUp
-  { isymbol  = 'O'
+treasureCache = ItemKind
+  { isymbol  = '0'
   , iname    = "treasure cache"
   , ifreq    = [("treasure cache", 1)]
   , iflavour = zipPlain [BrBlue]
+  , icount   = 1
+  , irarity  = [(1, 1)]
+  , iverbHit = "crash"
+  , iweight  = 10000
+  , idamage  = 0
+  , iaspects = [SetFlag Durable]
   , ieffects = [CreateItem CGround "common item" timerNone]
-  , idesc    = "Glittering gold, just waiting to be taken."
+  , idesc    = "Glittering treasure, just waiting to be taken."
+  , ikit     = []
   }
 treasureCacheTrap = ItemKind
   { isymbol  = '^'
@@ -100,39 +103,41 @@ treasureCacheTrap = ItemKind
   , iverbHit = "taint"
   , iweight  = 1000
   , idamage  = 0
-  , iaspects = []
-  , ieffects = [OneOf [ toOrganBad "blind" (40 + 1 `d` 10)
+  , iaspects = []  -- not Durable, springs at most once
+  , ieffects = [OneOf [ toOrganBad "blind" (10 + 1 `d` 10)
                       , RefillCalm (-99)
                       , Explode "focused concussion"
-                      , RefillCalm (-1), RefillCalm (-1), RefillCalm (-1)
-                      , RefillCalm (-1), RefillCalm (-1), RefillCalm (-1)
-                      , RefillCalm (-1) ]]
-  , ifeature = []  -- not Durable, springs at most once
+                      , RefillCalm (-1), RefillCalm (-1), RefillCalm (-1) ]]
   , idesc    = "It's a trap!"
   , ikit     = []
   }
 signboardExit = ItemKind
   { isymbol  = '?'
   , iname    = "inscription"
-  , ifreq    = [("signboard", 80)]
+  , ifreq    = [("signboard", 50)]
   , iflavour = zipPlain [BrMagenta]
   , icount   = 1
   , irarity  = [(1, 1)]
   , iverbHit = "whack"
   , iweight  = 10000
   , idamage  = 0
-  , iaspects = []
+  , iaspects = [SetFlag Durable]
   , ieffects = [Detect DetectExit 100]
-  , ifeature = [Durable]
   , idesc    = "Crude big arrows hastily carved by unknown hands."
   , ikit     = []
   }
 signboardEmbed = signboardExit
   { iname    = "notice"
-  , ifreq    = [("signboard", 20)]
+  , ifreq    = [("signboard", 50)]
   , ieffects = [Detect DetectEmbed 12]
   , idesc    = "The battered poster is untitled and unsigned."
   }
+signboardMerchandise = signboardExit
+  { iname    = "treasure map"
+  , ifreq    = [("signboard", 50)]
+  , ieffects = [Detect DetectLoot 20]
+  , idesc    = "In equal parts cryptic and promising."
+  }
 fireSmall = ItemKind
   { isymbol  = '%'
   , iname    = "small fire"
@@ -143,19 +148,18 @@ fireSmall = ItemKind
   , iverbHit = "burn"
   , iweight  = 10000
   , idamage  = 0
-  , iaspects = []
+  , iaspects = [SetFlag Durable]
   , ieffects = [Burn 1, Explode "single spark"]
-  , ifeature = [Durable]
   , idesc    = "A few small logs, burning brightly."
   , ikit     = []
   }
 fireBig = fireSmall
-  { isymbol  = 'O'
+  { isymbol  = '0'
   , iname    = "big fire"
   , ifreq    = [("big fire", 1)]
-  , ieffects = [ Burn 2, Explode "spark"
-               , CreateItem CInv "wooden torch" timerNone ]
-  , ifeature = [Durable]
+  , ieffects = [ Burn 2
+               , CreateItem CInv "wooden torch" timerNone
+               , Explode "spark" ]
   , idesc    = "Glowing with light and warmth."
   , ikit     = []
   }
@@ -169,11 +173,10 @@ frost = ItemKind
   , iverbHit = "burn"
   , iweight  = 10000
   , idamage  = 0
-  , iaspects = []
+  , iaspects = [SetFlag Durable]
   , ieffects = [ Burn 1  -- sensory ambiguity between hot and cold
                , RefillCalm 20  -- cold reason
-               , PushActor (ThrowMod 100 50) ]  -- slippery ice, 1 step, slow
-  , ifeature = [Durable]
+               , PushActor (ThrowMod 400 10 1) ]  -- slippery ice
   , idesc    = "Intricate patterns of shining ice."
   , ikit     = []
   }
@@ -187,14 +190,13 @@ rubble = ItemKind
   , iverbHit = "bury"
   , iweight  = 100000
   , idamage  = 0
-  , iaspects = []
+  , iaspects = [SetFlag Durable]
   , ieffects = [OneOf [ Explode "focused glass hail"
-                      , Summon "animal" $ 1 `dL` 2
+                      , Summon "mobile animal" $ 1 `dL` 2
                       , toOrganNoTimer "poisoned"
                       , CreateItem CGround "common item" timerNone
                       , RefillCalm (-1), RefillCalm (-1), RefillCalm (-1)
                       , RefillCalm (-1), RefillCalm (-1), RefillCalm (-1) ]]
-  , ifeature = [Durable]
   , idesc    = "Broken chunks of rock and glass."
   , ikit     = []
   }
@@ -208,16 +210,15 @@ doorwayTrapTemplate = ItemKind
   , iverbHit = "cripple"
   , iweight  = 10000
   , idamage  = 0
-  , iaspects = []
-  , ieffects = []
-  , ifeature = [HideAs "doorway trap unknown"]
+  , iaspects = [HideAs "doorway trap unknown"]
       -- not Durable, springs at most once
+  , ieffects = []
   , idesc    = "Just turn the handle..."
   , ikit     = []
   }
 doorwayTrap1 = doorwayTrapTemplate
   { ifreq    = [("doorway trap", 50)]
-  , ieffects = [toOrganBad "blind" $ (1 `dL` 4) * 10]
+  , ieffects = [toOrganBad "blind" $ (1 `dL` 4) * 5]
   -- , idesc    = ""
   }
 doorwayTrap2 = doorwayTrapTemplate
@@ -232,7 +233,7 @@ doorwayTrap3 = doorwayTrapTemplate
   }
 stairsUp = ItemKind
   { isymbol  = '<'
-  , iname    = "flight of steps"
+  , iname    = "flight"
   , ifreq    = [("staircase up", 1)]
   , iflavour = zipPlain [BrWhite]
   , icount   = 1
@@ -241,15 +242,13 @@ stairsUp = ItemKind
                         -- not when it's applied otherwise, e.g., from tile
   , iweight  = 100000
   , idamage  = 0
-  , iaspects = []
+  , iaspects = [ELabel "of steps", SetFlag Durable]
   , ieffects = [Ascend True]
-  , ifeature = [Durable]
   , idesc    = "Stairs that rise towards escape."
   , ikit     = []
   }
 stairsDown = stairsUp
   { isymbol  = '>'
-  , iname    = "flight of steps"
   , ifreq    = [("staircase down", 1)]
   , ieffects = [Ascend False]
   , idesc    = ""
@@ -259,8 +258,9 @@ escape = stairsUp
   , iname    = "way"
   , ifreq    = [("escape", 1)]
   , iflavour = zipPlain [BrYellow]
+  , iaspects = [SetFlag Durable]
   , ieffects = [Escape]
-  , idesc    = ""
+  , idesc    = "May this nightmare have an end?"
   }
 staircaseTrapUp = ItemKind
   { isymbol  = '^'
@@ -272,10 +272,9 @@ staircaseTrapUp = ItemKind
   , iverbHit = "buffet"
   , iweight  = 10000
   , idamage  = 0
-  , iaspects = []
-  , ieffects = [ Temporary "be caught in an updraft"
+  , iaspects = []  -- not Durable, springs at most once
+  , ieffects = [ VerbMsg "be caught in an updraft"
                , Teleport $ 3 + 1 `dL` 10 ]
-  , ifeature = []  -- not Durable, springs at most once
   , idesc    = "A hidden spring, to help the unwary soar."
   , ikit     = []
   }
@@ -285,7 +284,7 @@ staircaseTrapUp = ItemKind
 staircaseTrapDown = staircaseTrapUp
   { ifreq    = [("staircase trap down", 1)]
   , iverbHit = "open up under"
-  , ieffects = [ Temporary "tumble down the stairwell"
+  , ieffects = [ VerbMsg "tumble down the stairwell"
                , toOrganGood "drunk" (20 + 1 `d` 5) ]
   , idesc    = "A treacherous slab, to teach those who are too proud."
   }
@@ -299,13 +298,61 @@ pulpit = ItemKind
   , iverbHit = "ask"
   , iweight  = 10000
   , idamage  = 0
-  , iaspects = []
+  , iaspects = []  -- not Durable, springs at most once
   , ieffects = [ OneOf [ CreateItem CGround "any scroll" timerNone
                        , Detect DetectAll 20
                        , toOrganBad "defenseless" $ (1 `dL` 6) * 10
                        , toOrganGood "drunk" (20 + 1 `d` 5) ]
                , Explode "PhD defense question" ]
-  , ifeature = []  -- not Durable, springs at most once
   , idesc    = "A dark wood stand, where strange priests once preached."
   , ikit     = []
   }
+shallowWater = ItemKind
+  { isymbol  = '~'
+  , iname    = "shallow water"
+  , ifreq    = [("shallow water", 1)]
+  , iflavour = zipFancy [BrCyan]
+  , icount   = 1
+  , irarity  = [(1, 1)]
+  , iverbHit = "impede"
+  , iweight  = 10000
+  , idamage  = 0
+  , iaspects = [SetFlag Durable]
+  , ieffects = [ParalyzeInWater 2]
+  , idesc    = ""
+  , ikit     = []
+  }
+straightPath = ItemKind
+  { isymbol  = '.'
+  , iname    = "straight path"
+  , ifreq    = [("straight path", 1)]
+  , iflavour = zipFancy [BrRed]
+  , icount   = 1
+  , irarity  = [(1, 1)]
+  , iverbHit = "propel"
+  , iweight  = 10000
+  , idamage  = 0
+  , iaspects = [SetFlag Durable]
+  , ieffects = [InsertMove 2]
+  , idesc    = ""
+  , ikit     = []
+  }
+frozenGround = ItemKind
+  { isymbol  = '.'
+  , iname    = "shade"
+  , ifreq    = [("frozen ground", 1)]
+  , iflavour = zipFancy [BrBlue]
+  , icount   = 50  -- very thick ice and refreezes
+  , irarity  = [(1, 1)]
+  , iverbHit = "betray"
+  , iweight  = 10000
+  , idamage  = 0
+  , iaspects = [ELabel "of ice"]
+                 -- no Durable or some items would be impossible to pick up
+  , ieffects = [PushActor (ThrowMod 400 10 1)]
+                  -- the high speed represents gliding rather than flying
+                  -- and so no need to lift actor's weight off the ground;
+                  -- low linger comes from abrupt halt over normal surface
+  , idesc    = ""
+  , ikit     = []
+  }
