@@ -5,19 +5,19 @@ module Content.ModeKind
 
 import Prelude ()
 
-import Game.LambdaHack.Common.Prelude
-
-import qualified Data.IntMap.Strict as IM
+import Game.LambdaHack.Core.Prelude
 
 import Content.ModeKindPlayer
-import Game.LambdaHack.Common.Dice
+import Game.LambdaHack.Content.CaveKind (CaveKind)
 import Game.LambdaHack.Content.ModeKind
+import Game.LambdaHack.Core.Dice
+import Game.LambdaHack.Definition.Defs
 
 content :: [ModeKind]
 content =
-  [raid, brawl, shootout, escape, zoo, ambush, crawl, crawlEmpty, crawlSurvival, safari, safariSurvival, battle, battleSurvival, defense, defenseEmpty, screensaverRaid, screensaverBrawl, screensaverShootout, screensaverEscape, screensaverZoo, screensaverAmbush, screensaverCrawl, screensaverSafari]
+  [raid, brawl, shootout, hunt, escape, zoo, ambush, crawl, crawlEmpty, crawlSurvival, dig, see, safari, safariSurvival, battle, battleDefense, battleSurvival, defense, defenseEmpty, screensaverRaid, screensaverBrawl, screensaverShootout, screensaverHunt, screensaverEscape, screensaverZoo, screensaverAmbush, screensaverCrawl, screensaverSafari]
 
-raid,    brawl, shootout, escape, zoo, ambush, crawl, crawlEmpty, crawlSurvival, safari, safariSurvival, battle, battleSurvival, defense, defenseEmpty, screensaverRaid, screensaverBrawl, screensaverShootout, screensaverEscape, screensaverZoo, screensaverAmbush, screensaverCrawl, screensaverSafari :: ModeKind
+raid,    brawl, shootout, hunt, escape, zoo, ambush, crawl, crawlEmpty, crawlSurvival, dig, see, safari, safariSurvival, battle, battleDefense, battleSurvival, defense, defenseEmpty, screensaverRaid, screensaverBrawl, screensaverShootout, screensaverHunt, screensaverEscape, screensaverZoo, screensaverAmbush, screensaverCrawl, screensaverSafari :: ModeKind
 
 -- What other symmetric (two only-one-moves factions) and asymmetric vs crowd
 -- scenarios make sense (e.g., are good for a tutorial or for standalone
@@ -29,24 +29,28 @@ raid,    brawl, shootout, escape, zoo, ambush, crawl, crawlEmpty, crawlSurvival,
 -- sparse ranged at night: no, less fun than dense and if no reaction fire,
 --   just a camp fest or firing blindly
 -- dense ranged in daylight: no, less fun than at night with flares
--- crowd ranged: no, fish in a barel, less predictable and more fun inside
+-- crowd ranged: no, fish in a barrel, less predictable and more fun inside
 --   crawl, even without reaction fire
 
 raid = ModeKind  -- mini-crawl
   { msymbol = 'r'
-  , mname   = "raid (1)"
-  , mfreq   = [("raid", 1), ("campaign scenario", 1)]
+  , mname   = "solo raid (1)"
+  , mfreq   = [("solo", 1), ("raid", 1), ("campaign scenario", 1)]
   , mroster = rosterRaid
   , mcaves  = cavesRaid
+  , mendMsg = [ (Killed, "This expedition has gone wrong. However, scientific mind does not despair, but analyzes and corrects. Did you perchance awake one animal too many? Did you remember to try using all consumables at your disposal for your immediate survival? Did you choose a challenge with difficulty level within your means? Answer honestly, ponder wisely, experiment methodically.")
+              , (Defeated, "Regrettably, the other team snatched the grant, while you were busy contemplating natural phenomena. Science is a competitive sport, as sad as it sounds. It's not enough to make a discovery, you have to get there first.")
+              , (Escape, "You've got hold of the machine! Think of the hours of fun taking it apart and putting it back together again! That's a great first step on your quest to solve the typing problems of the world.") ]
   , mdesc   = "An incredibly advanced typing machine worth 100 gold is buried at the exit of this maze. Be the first to find it and fund a research team that makes typing accurate and dependable forever."
   }
 
 brawl = ModeKind  -- sparse melee in daylight, with shade for melee ambush
   { msymbol = 'k'
-  , mname   = "brawl (2)"
-  , mfreq   = [("brawl", 1), ("campaign scenario", 1)]
+  , mname   = "melee brawl (2)"
+  , mfreq   = [("melee", 1), ("brawl", 1), ("campaign scenario", 1)]
   , mroster = rosterBrawl
   , mcaves  = cavesBrawl
+  , mendMsg = []
   , mdesc   = "Your engineering team disagrees over a drink with some gentlemen scientists about premises of a relative completeness theorem and there's only one way to settle that. Remember to keep your party together, or the opposing team might be tempted to gang upon a solitary disputant."
   }
 
@@ -54,35 +58,48 @@ brawl = ModeKind  -- sparse melee in daylight, with shade for melee ambush
 -- behind a bush and others hiding in mist. If no suitable bushes,
 -- fire once and flee into mist or behind cover. Then whomever is out of LOS
 -- range or inside mist can shoot at the last seen enemy locations,
--- adjusting and according to ounds and incoming missile trajectories.
--- If the scount can't find bushes or glass building to set a lookout,
--- the other team member are more spotters and guardians than snipers
+-- adjusting aim according to sounds and incoming missile trajectories.
+-- If the scout can't find bushes or glass building to set a lookout,
+-- the other team members are more spotters and guardians than snipers
 -- and that's their only role, so a small party makes sense.
 shootout = ModeKind  -- sparse ranged in daylight
   { msymbol = 's'
-  , mname   = "shootout (3)"
-  , mfreq   = [("shootout", 1), ("campaign scenario", 1)]
+  , mname   = "foggy shootout (3)"
+  , mfreq   = [("foggy", 1), ("shootout", 1), ("campaign scenario", 1)]
   , mroster = rosterShootout
   , mcaves  = cavesShootout
+  , mendMsg = []
   , mdesc   = "Whose arguments are most striking and whose ideas fly fastest? Let's scatter up, attack the problems from different angles and find out. (To display the trajectory of any soaring entity, point it with the crosshair in aiming mode.)"
   }
 
+hunt = ModeKind  -- melee vs ranged with reaction fire in daylight
+  { msymbol = 'h'
+  , mname   = "perilous hunt (4)"
+  , mfreq   = [("perilous", 1), ("hunt", 1), ("campaign scenario", 1)]
+  , mroster = rosterHunt
+  , mcaves  = cavesHunt
+  , mendMsg = []
+  , mdesc   = "Who is the hunter and who is the prey?"
+  }
+
 escape = ModeKind  -- asymmetric ranged and stealth race at night
   { msymbol = 'e'
-  , mname   = "escape (4)"
-  , mfreq   = [("escape", 1), ("campaign scenario", 1)]
+  , mname   = "night escape (5)"
+  , mfreq   = [("night", 1), ("escape", 1), ("campaign scenario", 1)]
   , mroster = rosterEscape
   , mcaves  = cavesEscape
+  , mendMsg = []
   , mdesc   = "Dwelling into dark matters is dangerous, so avoid the crowd of firebrand disputants, catch any gems of thought, find a way out and bring back a larger team to shed new light on the field."
   }
 
 zoo = ModeKind  -- asymmetric crowd melee at night
   { msymbol = 'b'
-  , mname   = "zoo (5)"
-  , mfreq   = [("zoo", 1), ("campaign scenario", 1)]
+  , mname   = "burning zoo (6)"
+  , mfreq   = [("burning", 1), ("zoo", 1), ("campaign scenario", 1)]
   , mroster = rosterZoo
   , mcaves  = cavesZoo
-  , mdesc   = "The heat of the dispute reaches the nearby Wonders of Science and Nature exhibition, igniting greenery, nets and cages. Crazed animals must be prevented from ruining precious scientific equipment and setting back the otherwise fruitful exchange of ideas."
+  , mendMsg = []
+  , mdesc   = "The heat of the dispute reaches the nearby Wonders of Science and Nature exhibition, igniting greenery, nets and cages. Crazed animals must be dissuaded from ruining precious scientific equipment and setting back the otherwise fruitful exchange of ideas."
   }
 
 -- The tactic is to sneak in the dark, highlight enemy with thrown torches
@@ -93,41 +110,66 @@ zoo = ModeKind  -- asymmetric crowd melee at night
 -- For now, while we have no shooters with timeout, massive ranged battles
 -- without reaction fire don't make sense, because then usually only one hero
 -- shoots (and often also scouts) and others just gather ammo.
-ambush = ModeKind  -- dense ranged with reaction fire at night
+ambush = ModeKind  -- dense ranged with reaction fire vs melee at night
   { msymbol = 'm'
-  , mname   = "ambush (6)"
-  , mfreq   = [("ambush", 1), ("campaign scenario", 1)]
+  , mname   = "ranged ambush (7)"
+  , mfreq   = [("ranged", 1), ("ambush", 1), ("campaign scenario", 1)]
   , mroster = rosterAmbush
   , mcaves  = cavesAmbush
-  , mdesc   = "Prevent hijacking of your ideas at all cost! Be stealthy, be aggressive. Fast execution is what makes or breaks a creative team."
+  , mendMsg = []
+  , mdesc   = "Prevent hijacking of your ideas at all cost! Be stealthy, be observant, be aggressive. Fast execution is what makes or breaks a creative team."
   }
 
 crawl = ModeKind
   { msymbol = 'c'
-  , mname   = "crawl (long)"
-  , mfreq   = [("crawl", 1), ("campaign scenario", 1)]
+  , mname   = "deep crawl (long)"
+  , mfreq   = [("deep", 1), ("crawl", 1), ("campaign scenario", 1)]
   , mroster = rosterCrawl
   , mcaves  = cavesCrawl
+  , mendMsg = [ (Killed, "To think that followers of science and agents of enlightenment would earn death as their reward! Where did we err in our ways? Perhaps nature should not have been disturbed so brashly and the fell beasts woken up from their slumber so eagerly? Perhaps the gathered items should have been used for scientific experiments on the spot rather than hoarded as if of base covetousness? Or perhaps the challenge, chosen freely but without the foreknowledge of the grisly difficulty, was insurmountable and forlorn from the start, despite the enormous power of educated reason at out disposal?")
+              , (Escape, "It's better to live to tell the tale than to choke on more than one can swallow. There was no more exquisite cultural artifacts and glorious scientific wonders in these forbidding tunnels anyway. Or were there?") ]
   , mdesc   = "Enjoy the peaceful seclusion of these cold austere tunnels, but don't let wanton curiosity, greed and the ever-creeping abstraction madness keep you down there for too long. If you find survivors (whole or perturbed or segmented) of the past scientific missions, exercise extreme caution and engage or ignore at your discretion."
   }
 
-safari = ModeKind  -- easter egg available only via screensaver
+safari = ModeKind  -- Easter egg available only via screensaver
   { msymbol = 'f'
   , mname   = "safari"
   , mfreq   = [("safari", 1)]
   , mroster = rosterSafari
   , mcaves  = cavesSafari
+  , mendMsg = []
   , mdesc   = "\"In this enactment you'll discover the joys of hunting the most exquisite of Earth's flora and fauna, both animal and semi-intelligent. Exit at the bottommost level.\" This is a drama script recovered from a monster nest debris."
   }
 
 -- * Testing modes
 
+dig = ModeKind
+  { msymbol = 'd'
+  , mname   = "dig"
+  , mfreq   = [("dig", 1)]
+  , mroster = rosterCrawlEmpty
+  , mcaves  = cavesDig
+  , mendMsg = []
+  , mdesc   = "Delve deeper!"
+  }
+
+see = ModeKind
+  { msymbol = 'a'
+  , mname   = "see"
+  , mfreq   = [("see", 1)]
+  , mroster = rosterCrawlEmpty
+  , mcaves  = cavesSee
+  , mendMsg = []
+  , mdesc   = "See all!"
+  }
+
 crawlEmpty = ModeKind
   { msymbol = 'c'
   , mname   = "crawl empty"
   , mfreq   = [("crawl empty", 1)]
   , mroster = rosterCrawlEmpty
   , mcaves  = cavesCrawl
+  , mendMsg = []
   , mdesc   = "Enjoy the free space."
   }
 
@@ -137,6 +179,7 @@ crawlSurvival = ModeKind
   , mfreq   = [("crawl survival", 1)]
   , mroster = rosterCrawlSurvival
   , mcaves  = cavesCrawl
+  , mendMsg = []
   , mdesc   = "Lure the human intruders deeper and deeper."
   }
 
@@ -146,6 +189,7 @@ safariSurvival = ModeKind
   , mfreq   = [("safari survival", 1)]
   , mroster = rosterSafariSurvival
   , mcaves  = cavesSafari
+  , mendMsg = []
   , mdesc   = "In this enactment you'll discover the joys of being hunted among the most exquisite of Earth's flora and fauna, both animal and semi-intelligent."
   }
 
@@ -155,16 +199,28 @@ battle = ModeKind
   , mfreq   = [("battle", 1)]
   , mroster = rosterBattle
   , mcaves  = cavesBattle
+  , mendMsg = []
   , mdesc   = "Odds are stacked against those that unleash the horrors of abstraction."
   }
 
+battleDefense = ModeKind
+  { msymbol = 'f'
+  , mname   = "battle defense"
+  , mfreq   = [("battle defense", 1)]
+  , mroster = rosterBattleDefense
+  , mcaves  = cavesBattle
+  , mendMsg = []
+  , mdesc   = "Odds are stacked for those that breathe mathematics."
+  }
+
 battleSurvival = ModeKind
   { msymbol = 'i'
   , mname   = "battle survival"
   , mfreq   = [("battle survival", 1)]
   , mroster = rosterBattleSurvival
   , mcaves  = cavesBattle
-  , mdesc   = "Odds are stacked for those that breathe mathematics."
+  , mendMsg = []
+  , mdesc   = "Odds are stacked for those that ally with the strongest."
   }
 
 defense = ModeKind  -- perhaps a real scenario in the future
@@ -173,6 +229,7 @@ defense = ModeKind  -- perhaps a real scenario in the future
   , mfreq   = [("defense", 1)]
   , mroster = rosterDefense
   , mcaves  = cavesCrawl
+  , mendMsg = []
   , mdesc   = "Don't let human interlopers defile your abstract secrets and flee unpunished!"
   }
 
@@ -182,6 +239,7 @@ defenseEmpty = ModeKind
   , mfreq   = [("defense empty", 1)]
   , mroster = rosterDefenseEmpty
   , mcaves  = cavesCrawl
+  , mendMsg = []
   , mdesc   = "Lord over."
   }
 
@@ -196,7 +254,7 @@ screensave auto r =
 
 screensaverRaid = raid
   { mname   = "auto-raid (1)"
-  , mfreq   = [("starting", 1), ("starting JS", 1), ("no confirms", 1)]
+  , mfreq   = [("insert coin", 1), ("no confirms", 1)]
   , mroster = screensave (AutoLeader False False) rosterRaid
   }
 
@@ -208,24 +266,30 @@ screensaverBrawl = brawl
 
 screensaverShootout = shootout
   { mname   = "auto-shootout (3)"
-  , mfreq   = [("starting", 1), ("starting JS", 1), ("no confirms", 1)]
+  , mfreq   = [("insert coin", 1), ("no confirms", 1)]
   , mroster = screensave (AutoLeader False False) rosterShootout
   }
 
+screensaverHunt = hunt
+  { mname   = "auto-hunt (4)"
+  , mfreq   = [("insert coin", 1), ("no confirms", 1)]
+  , mroster = screensave (AutoLeader False False) rosterHunt
+  }
+
 screensaverEscape = escape
-  { mname   = "auto-escape (4)"
-  , mfreq   = [("starting", 1), ("starting JS", 1), ("no confirms", 1)]
+  { mname   = "auto-escape (5)"
+  , mfreq   = [("insert coin", 1), ("no confirms", 1)]
   , mroster = screensave (AutoLeader False False) rosterEscape
   }
 
 screensaverZoo = zoo
-  { mname   = "auto-zoo (5)"
+  { mname   = "auto-zoo (6)"
   , mfreq   = [("no confirms", 1)]
   , mroster = screensave (AutoLeader False False) rosterZoo
   }
 
 screensaverAmbush = ambush
-  { mname   = "auto-ambush (6)"
+  { mname   = "auto-ambush (7)"
   , mfreq   = [("no confirms", 1)]
   , mroster = screensave (AutoLeader False False) rosterAmbush
   }
@@ -238,18 +302,18 @@ screensaverCrawl = crawl
 
 screensaverSafari = safari
   { mname   = "auto-safari"
-  , mfreq   = [("starting", 1), ("starting JS", 1), ("no confirms", 1)]
+  , mfreq   = [("insert coin", 1), ("no confirms", 1)]
   , mroster = -- changing leader by client needed, because of TFollow
               screensave (AutoLeader False True) rosterSafari
   }
 
-rosterRaid, rosterBrawl, rosterShootout, rosterEscape, rosterZoo, rosterAmbush, rosterCrawl, rosterCrawlEmpty, rosterCrawlSurvival, rosterSafari, rosterSafariSurvival, rosterBattle, rosterBattleSurvival, rosterDefense, rosterDefenseEmpty :: Roster
+rosterRaid, rosterBrawl, rosterShootout, rosterHunt, rosterEscape, rosterZoo, rosterAmbush, rosterCrawl, rosterCrawlEmpty, rosterCrawlSurvival, rosterSafari, rosterSafariSurvival, rosterBattle, rosterBattleDefense, rosterBattleSurvival, rosterDefense, rosterDefenseEmpty :: Roster
 
 rosterRaid = Roster
-  { rosterList = [ ( playerHero {fhiCondPoly = hiRaid}
+  { rosterList = [ ( playerHero {fhiCondPoly = hiHeroShort}
                    , [(-2, 1, "hero")] )
                  , ( playerAntiHero { fname = "Indigo Founder"
-                                    , fhiCondPoly = hiRaid }
+                                    , fhiCondPoly = hiHeroShort }
                    , [(-2, 1, "hero")] )
                  , ( playerAnimal  -- starting over escape
                    , [(-2, 2, "animal")] )
@@ -262,11 +326,11 @@ rosterRaid = Roster
 
 rosterBrawl = Roster
   { rosterList = [ ( playerHero { fcanEscape = False
-                                , fhiCondPoly = hiDweller }
+                                , fhiCondPoly = hiHeroMedium }
                    , [(-3, 3, "hero")] )
                  , ( playerAntiHero { fname = "Indigo Researcher"
                                     , fcanEscape = False
-                                    , fhiCondPoly = hiDweller }
+                                    , fhiCondPoly = hiHeroMedium }
                    , [(-3, 3, "hero")] )
                  , (playerHorror, []) ]
   , rosterEnemy = [ ("Explorer", "Indigo Researcher")
@@ -280,11 +344,11 @@ rosterBrawl = Roster
 -- scout to counter the stealthy advance.
 rosterShootout = Roster
   { rosterList = [ ( playerHero { fcanEscape = False
-                                , fhiCondPoly = hiDweller }
+                                , fhiCondPoly = hiHeroMedium }
                    , [(-5, 1, "scout hero"), (-5, 2, "ranger hero")] )
                  , ( playerAntiHero { fname = "Indigo Researcher"
                                     , fcanEscape = False
-                                    , fhiCondPoly = hiDweller }
+                                    , fhiCondPoly = hiHeroMedium }
                    , [(-5, 1, "scout hero"), (-5, 2, "ranger hero")] )
                  , (playerHorror, []) ]
   , rosterEnemy = [ ("Explorer", "Indigo Researcher")
@@ -292,14 +356,28 @@ rosterShootout = Roster
                   , ("Indigo Researcher", "Horror Den") ]
   , rosterAlly = [] }
 
+rosterHunt = Roster
+  { rosterList = [ ( playerHero { fcanEscape = False
+                                , fhiCondPoly = hiHeroMedium }
+                   , [(-6, 7, "soldier hero")] )
+                 , ( playerAntiHero { fname = "Indigo Researcher"
+                                    , fcanEscape = False
+                                    , fhiCondPoly = hiHeroMedium }
+                   , [(-6, 1, "scout hero"), (-6, 6, "ambusher hero")] )
+                 , (playerHorror, []) ]
+  , rosterEnemy = [ ("Explorer", "Indigo Researcher")
+                  , ("Explorer", "Horror Den")
+                  , ("Indigo Researcher", "Horror Den") ]
+  , rosterAlly = [] }
+
 rosterEscape = Roster
-  { rosterList = [ ( playerHero {fhiCondPoly = hiEscapist}
+  { rosterList = [ ( playerHero {fhiCondPoly = hiHeroMedium}
                    , [(-7, 1, "scout hero"), (-7, 2, "escapist hero")] )
                  , ( playerAntiHero { fname = "Indigo Researcher"
                                     , fcanEscape = False  -- start on escape
                                     , fneverEmpty = False  -- loot after killing
-                                    , fhiCondPoly = hiDweller }
-                   , [(-7, 1, "scout hero"), (-7, 7, "ambusher hero")] )
+                                    , fhiCondPoly = hiHeroMedium }
+                   , [(-7, 1, "scout hero"), (-7, 6, "ambusher hero")] )
                  , (playerHorror, []) ]
   , rosterEnemy = [ ("Explorer", "Indigo Researcher")
                   , ("Explorer", "Horror Den")
@@ -308,7 +386,7 @@ rosterEscape = Roster
 
 rosterZoo = Roster
   { rosterList = [ ( playerHero { fcanEscape = False
-                                , fhiCondPoly = hiDweller }
+                                , fhiCondPoly = hiHeroLong }
                    , [(-8, 5, "soldier hero")] )
                  , ( playerAnimal {fneverEmpty = True}
                    , [(-8, 100, "mobile animal")] )
@@ -319,12 +397,12 @@ rosterZoo = Roster
 
 rosterAmbush = Roster
   { rosterList = [ ( playerHero { fcanEscape = False
-                                , fhiCondPoly = hiDweller }
+                                , fhiCondPoly = hiHeroMedium }
                    , [(-9, 1, "scout hero"), (-9, 5, "ambusher hero")] )
                  , ( playerAntiHero { fname = "Indigo Researcher"
                                     , fcanEscape = False
-                                    , fhiCondPoly = hiDweller }
-                   , [(-9, 1, "scout hero"), (-9, 5, "ambusher hero")] )
+                                    , fhiCondPoly = hiHeroMedium }
+                   , [(-9, 12, "soldier hero")] )
                  , (playerHorror, []) ]
   , rosterEnemy = [ ("Explorer", "Indigo Researcher")
                   , ("Explorer", "Horror Den")
@@ -348,14 +426,12 @@ rosterCrawl = Roster
 rosterCrawlEmpty = Roster
   { rosterList = [ ( playerHero
                    , [(-1, 1, "hero")] )
-                 , (playerHorror, []) ]  -- for summoned monsters
+                 , (playerHorror, []) ]  -- for spawned and summoned monsters
   , rosterEnemy = []
   , rosterAlly = [] }
 
 rosterCrawlSurvival = rosterCrawl
-  { rosterList = [ ( playerHero { fleaderMode =
-                                    LeaderAI $ AutoLeader True False
-                                , fhasUI = False }
+  { rosterList = [ ( playerAntiHero
                    , [(-1, 3, "hero")] )
                  , ( playerMonster
                    , [(-4, 1, "scout monster"), (-4, 3, "monster")] )
@@ -370,11 +446,11 @@ rosterSafari = Roster
   { rosterList = [ ( playerMonsterTourist
                    , [(-4, 15, "monster")] )
                  , ( playerHunamConvict
-                   , [(-4, 3, "civilian")] )
+                   , [(-4, 2, "civilian")] )
                  , ( playerAnimalMagnificent
-                   , [(-7, 20, "mobile animal")] )
+                   , [(-7, 15, "mobile animal")] )
                  , ( playerAnimalExquisite  -- start on escape
-                   , [(-10, 30, "mobile animal")] ) ]
+                   , [(-10, 20, "mobile animal")] ) ]
   , rosterEnemy = [ ("Monster Tourist Office", "Hunam Convict")
                   , ( "Monster Tourist Office"
                     , "Animal Magnificent Specimen Variety" )
@@ -403,7 +479,7 @@ rosterSafariSurvival = rosterSafari
 
 rosterBattle = Roster
   { rosterList = [ ( playerHero { fcanEscape = False
-                                , fhiCondPoly = hiDweller }
+                                , fhiCondPoly = hiHeroLong }
                    , [(-5, 5, "soldier hero")] )
                  , ( playerMonster {fneverEmpty = True}
                    , [(-5, 35, "mobile monster")] )
@@ -413,9 +489,22 @@ rosterBattle = Roster
                   , ("Explorer", "Animal Kingdom") ]
   , rosterAlly = [("Monster Hive", "Animal Kingdom")] }
 
+rosterBattleDefense = rosterBattle
+  { rosterList = [ ( playerHero { fcanEscape = False
+                                , fhiCondPoly = hiHeroLong
+                                , fleaderMode =
+                                    LeaderAI $ AutoLeader False False
+                                , fhasUI = False }
+                   , [(-5, 5, "soldier hero")] )
+                 , ( playerMonster { fneverEmpty = True
+                                   , fhasUI = True }
+                   , [(-5, 35, "mobile monster")] )
+                 , ( playerAnimal {fneverEmpty = True}
+                   , [(-5, 30, "mobile animal")] ) ] }
+
 rosterBattleSurvival = rosterBattle
   { rosterList = [ ( playerHero { fcanEscape = False
-                                , fhiCondPoly = hiDweller
+                                , fhiCondPoly = hiHeroLong
                                 , fleaderMode =
                                     LeaderAI $ AutoLeader False False
                                 , fhasUI = False }
@@ -438,34 +527,59 @@ rosterDefense = rosterCrawl
 rosterDefenseEmpty = rosterCrawl
   { rosterList = [ ( playerAntiMonster {fneverEmpty = True}
                    , [(-4, 1, "scout monster")] )
-                 , (playerHorror, []) ]  -- for summoned animals
+                 , (playerHorror, []) ]  -- for spawned and summoned animals
   , rosterEnemy = []
   , rosterAlly = [] }
 
-cavesRaid, cavesBrawl, cavesShootout, cavesEscape, cavesZoo, cavesAmbush, cavesCrawl, cavesSafari, cavesBattle :: Caves
+cavesRaid, cavesBrawl, cavesShootout, cavesHunt, cavesEscape, cavesZoo, cavesAmbush, cavesCrawl, cavesDig, cavesSee, cavesSafari, cavesBattle :: Caves
 
-cavesRaid = IM.fromList [(-2, "caveRaid")]
+cavesRaid = [([-2], ["caveRaid"])]
 
-cavesBrawl = IM.fromList [(-3, "caveBrawl")]
+cavesBrawl = [([-3], ["caveBrawl"])]
 
-cavesShootout = IM.fromList [(-5, "caveShootout")]
+cavesShootout = [([-5], ["caveShootout"])]
 
-cavesEscape = IM.fromList [(-7, "caveEscape")]
+cavesHunt = [([-6], ["caveHunt"])]
 
-cavesZoo = IM.fromList [(-8, "caveZoo")]
+cavesEscape = [([-7], ["caveEscape"])]
 
-cavesAmbush = IM.fromList [(-9, "caveAmbush")]
+cavesZoo = [([-8], ["caveZoo"])]
 
-cavesCrawl = IM.fromList $
-  [ (-1, "outermost")
-  , (-2, "shallow random 2")
-  , (-3, "caveEmpty") ]
-  ++ zip [-4, -5] (repeat "default random")
-  ++ zip [-6, -7, -8, -9] (repeat "deep random")
-  ++ [(-10, "caveNoise2")]
+cavesAmbush = [([-9], ["caveAmbush"])]
 
-cavesSafari = IM.fromList [ (-4, "caveSafari1")
-                          , (-7, "caveSafari2")
-                          , (-10, "caveSafari3") ]
+listCrawl :: [([Int], [GroupName CaveKind])]
+listCrawl =
+  [ ([-1], ["caveOutermost"])
+  , ([-2], ["caveShallowRogue"])
+  , ([-3], ["caveEmpty"])
+  , ([-4, -5, -6], ["default random", "caveRogue", "caveArena"])
+  , ([-7, -8], ["caveRogue", "caveSmoking"])
+  , ([-9], ["caveLaboratory"])
+  , ([-10], ["caveMine"]) ]
 
-cavesBattle = IM.fromList [(-5, "caveBattle")]
+cavesCrawl = listCrawl
+
+renumberCaves :: Int -> ([Int], [GroupName CaveKind])
+              -> ([Int], [GroupName CaveKind])
+renumberCaves offset (ns, l) = (map (+ offset) ns, l)
+
+cavesDig = concat $ zipWith (map . renumberCaves)
+                            [0, -10 ..]
+                            (replicate 100 listCrawl)
+
+cavesSee = let numberCaves n c = ([n], [c])
+           in zipWith numberCaves [-1, -2 ..]
+              $ concatMap (replicate 8) allCaves
+
+allCaves :: [GroupName CaveKind]
+allCaves =
+  [ "caveRaid", "caveBrawl", "caveShootout", "caveHunt", "caveEscape", "caveZoo"
+  , "caveAmbush"
+  , "caveRogue", "caveLaboratory", "caveEmpty", "caveArena", "caveSmoking"
+  , "caveNoise", "caveMine" ]
+
+cavesSafari = [ ([-4], ["caveSafari1"])
+              , ([-7], ["caveSafari2"])
+              , ([-10], ["caveSafari3"]) ]
+
+cavesBattle = [([-5], ["caveBattle"])]
