@@ -6,32 +6,33 @@ module Main
 
 import Prelude ()
 
-import Game.LambdaHack.Common.Prelude
+import Game.LambdaHack.Core.Prelude
 
-import           Control.Concurrent
 import           Control.Concurrent.Async
 import qualified Control.Exception as Ex
 import qualified Options.Applicative as OA
 import           System.Exit
+import qualified System.IO as SIO
 
 #ifndef USE_JSFILE
 import qualified GHC.IO.Handle
 import           System.FilePath
-import qualified System.IO as SIO
 
 import Game.LambdaHack.Common.File (tryCreateDir)
+import Game.LambdaHack.Common.Misc
 #endif
 
-import Game.LambdaHack.Common.Misc
 import Game.LambdaHack.Server (serverOptionsPI)
 
-import Implementation.TieKnot
+import TieKnot
 
 -- | Parse commandline options, tie the engine, content and clients knot,
 -- run the game and handle exit.
 main :: IO ()
 main = do
-  -- This may be broken with JS and also bloats the outcome file, so disabled.
+  -- This test is faulty with JS, because it reports the browser console
+  -- is not a terminal, but then we can't open files to contain the logs.
+  -- Also it bloats the outcome JS file, so disabled.
 #ifndef USE_JSFILE
   -- Special case hack, when the game is started not on a console.
   -- Without this, any attempt to output on stdout crashes a Windows exe
@@ -47,24 +48,14 @@ main = do
     fstderr <- SIO.openFile (dataDir </> "stderr.txt") SIO.WriteMode
     GHC.IO.Handle.hDuplicateTo fstdout SIO.stdout
     GHC.IO.Handle.hDuplicateTo fstderr SIO.stderr
-  let fillWorkaround =
-        -- Set up void workaround if nothing specific required.
-        void $ tryPutMVar workaroundOnMainThreadMVar $ return ()
+#else
+  -- Work around display of one character per line.
+  SIO.hSetBuffering SIO.stderr SIO.LineBuffering
 #endif
   -- Fail here, not inside server code, so that savefiles are not removed,
   -- because they are not the source of the failure.
   !serverOptions <- OA.execParser serverOptionsPI
-  -- Avoid the bound thread that would slow down the communication.
-  a <- async $ tieKnot serverOptions
-#ifndef USE_JSFILE
-               `Ex.finally` fillWorkaround
-  -- Run a (possibly void) workaround. It's needed for architectures/frontends
-  -- that need to perform some actions on the main thread
-  -- (not just any bound thread), e.g., newer OS X drawing with SDL2.
-  workaround <- takeMVar workaroundOnMainThreadMVar
-  workaround
-#endif
-  resOrEx <- waitCatch a
+  resOrEx :: Either Ex.SomeException () <- Ex.try $ tieKnot serverOptions
   let unwrapEx e =
 #if MIN_VERSION_async(2,2,1)
         case Ex.fromException e of
